fulcrumPosition = new THREE.Vector3(0, 0, 15) # 紐の支点の位置
_radius = 0.3 # ボールの半径
_mass = 0.1   # ボールの質量
_position = new THREE.Vector3(2, 0, 3) # ボールの初期位置
_velocity = new THREE.Vector3(0, 0, 0) # ボールの初速度

# トラックボール (マウスで画面を回転させるため仕組み) の中心点
trackballTarget = new THREE.Vector3(0, 0, 2)

#--------------------------------------------------------------------------------
class @Main
  showForceVectors: true
  showVelocityVector: true
  showTrail: true
  timeScale: 1

  ball = null
  renderer = scene = container = null
  camera = trackball = stats = null
  directionalLight = ambientLight = null
  ballMesh = velocityVector = forceVector = fulcrumMesh = trail = strap = null
  forceVector1 = null
  forceVector2 = null
  clock = deltaTime = null

  constructor: ->
    @init()
    new GUI(this, ball).create()

  start: ->
    @calcPhysics()
    render()
    @animate()

  # 初期化
  init: ->
    @initThree()
    initCamera()
    initLight()
    initObject()
    clock = new THREE.Clock()

  # Three.js の初期化
  initThree: ->
    container = document.createElement 'div'
    document.body.appendChild container

    #----------------------------------------
    renderer = new THREE.WebGLRenderer(antialias: true)
    unless renderer? then alert('Three.js の初期化に失敗しました')
    renderer.setSize(window.innerWidth, window.innerHeight)
    container.appendChild(renderer.domElement)

    renderer.setClearColor(0xe1fcff, 1.0)
    renderer.shadowMapEnabled = true

    #----------------------------------------
    # Stats
    stats = new Stats()
    stats.domElement.style.position = 'absolute'
    stats.domElement.style.bottom = '0px'
    container.appendChild stats.domElement

    #----------------------------------------
    scene = new THREE.Scene()

    # フォグ (霧)
    #scene.fog = new THREE.Fog(0xffffff, 10, 40)

    #----------------------------------------
    window.addEventListener('resize', @onWindowResize, false)

  # カメラの初期化
  initCamera = ->
    camera = new THREE.PerspectiveCamera(40, window.innerWidth/window.innerHeight, 1, 10000)
    camera.position.set(0, -15, 6)
    camera.up.set(0, 0, 1)
    camera.lookAt(x: 0, y: 0, z: 50) #トラックボール利用時は自動的に無効

    trackball = new THREE.TrackballControls(camera, container)
    #trackball.screen.width = container.clientWidth
    #trackball.screen.height = container.clientHeight
    #trackball.screen.offsetLeft = container.getBoundingClientRect().left
    #trackball.screen.offsetTop = container.getBoundingClientRect().top
    trackball.rotateSpeed = 2.0
    trackball.zoomSpeed = 1.0
    trackball.panSpeed = 1.0
    trackball.noRotate = false
    trackball.noZoom = false
    trackball.noPan = false
    trackball.staticMoving = true
    trackball.dynamicDampingFactor = 0.3
    trackball.addEventListener('change', render)
    trackball.target = trackballTarget

  # 光源の初期化
  initLight = ->
    # directional light (平行光源)
    directionalLight = new THREE.DirectionalLight(0xffffff, 1.0, 0)
    directionalLight.position.set(0, 0, 50)
    directionalLight.castShadow = true
    scene.add directionalLight

    directionalLight.shadowMapWidth  = 2048
    directionalLight.shadowMapHeight = 2048

    directionalLight.shadowCameraRight  = 10
    directionalLight.shadowCameraLeft   = -10
    directionalLight.shadowCameraTop    = 10
    directionalLight.shadowCameraBottom = -10
    directionalLight.shadowCameraNear = 10
    directionalLight.shadowCameraFar = 100

    #directionalLight.shadowCameraVisible = true

    # ambient light (環境光源)
    ambientLight = new THREE.AmbientLight(0x777777)
    scene.add ambientLight

  toggleVelocityVector: ->
    scene.remove velocityVector
    if @showVelocityVector
      scene.add velocityVector

  toggleForceVectors: ->
    scene.remove forceVector
    scene.remove forceVector1
    scene.remove forceVector2
    if @showForceVectors
      scene.add forceVector
      scene.add forceVector1
      scene.add forceVector2

  toggleTrail: ->
    scene.remove trail
    if @showTrail
      scene.add trail

  clearTrail: ->
    trail.clear()

  # 描画するオブジェクトの初期化
  initObject = ->
    #------------------------------------------------------------
    # Ball を作成
    ball = new Ball(
      radius: _radius
      mass: _mass
      position: _position
      velocity: _velocity
      fulcrumPosition: fulcrumPosition
    )

    geometry = new THREE.SphereGeometry(ball.radius, 20, 20)
    material = new THREE.MeshLambertMaterial(color: 0xff0000, ambient: 0x880000)
    ballMesh = new THREE.Mesh(geometry, material)
    ballMesh.position = ball.position
    ballMesh.castShadow = true
    scene.add ballMesh

    #------------------------------------------------------------
    # 速度ベクトルを作成
    velocityVector = new THREE.ArrowHelper(
      ball.velocity.clone().normalize(),
      new THREE.Vector3(),
      ball.velocity.length()/4,
      0x0033ff
    )
    velocityVector.position = ball.position
    scene.add velocityVector

    #------------------------------------------------------------
    # 力ベクトルを作成
    forceVector = new THREE.ArrowHelper(
      ball.force.clone().normalize(),
      new THREE.Vector3(),
      ball.force.length(),
      0xff3300
    )
    forceVector.position = ball.position
    scene.add forceVector

    forceVector1 = new THREE.ArrowHelper(
      ball.forces.tension.clone().normalize(),
      new THREE.Vector3(),
      ball.forces.tension.length(),
      0xff3300
    )
    forceVector1.position = ball.position
    scene.add forceVector1
    forceVector2 = new THREE.ArrowHelper(
      ball.forces.gravity.clone().normalize(),
      new THREE.Vector3(),
      ball.forces.gravity.length(),
      0xff3300
    )
    forceVector2.position = ball.position
    scene.add forceVector2

    #------------------------------------------------------------
    # Fulcrum
    #geometry = new THREE.SphereGeometry(1, 10, 10)
    #material = new THREE.MeshLambertMaterial(color: 0xffff00, ambient: 0xffff00)
    #fulcrumMesh = new THREE.Mesh(geometry, material)
    #fulcrumMesh.position.copy fulcrumPosition
    #scene.add fulcrumMesh

    #------------------------------------------------------------
    # Floor
    floor = new Floor(20, 1, true)
    scene.add floor

    #------------------------------------------------------------
    # Human
    human = new Human()
    human.position = new THREE.Vector3(3,0,0)
    #human.scale.set(10,10,10)
    scene.add human

    #------------------------------------------------------------
    # Axes
    axes = new THREE.AxisHelper(10)
    axes.position.setZ 0.01
    scene.add axes

    #------------------------------------------------------------
    # Trail
    trail = new Trail(ballMesh, 1000, _position)
    scene.add trail

    #------------------------------------------------------------
    # Strap
    geometry = new THREE.Geometry()
    geometry.vertices = [fulcrumPosition, ball.position]
    strap = new THREE.Line(
      geometry,
      new THREE.LineBasicMaterial(color: 0x555555, linewidth: 2),
      THREE.LinePieces
    )
    scene.add strap

  #--------------------------------------------------------------------------------
  # アニメーションのオン/オフ
  toAnimate = false
  toggleAnimation: ->
    toAnimate = !toAnimate

  # アニメーション・ステップ
  renderedThisFrame = false
  animate: =>
    deltaTime = clock.getDelta()

    renderedThisFrame = false
    requestAnimationFrame(@animate)
    stats.update()
    trackball.update()
    if toAnimate
      @calcPhysics()
      render()

  # 描画の実行
  render = ->
    return if renderedThisFrame
    renderedThisFrame = true
    #renderer.clear()
    renderer.render(scene, camera)

  setTimeScale: (scale) ->
    @timeScale = scale

  calcPhysics: ->
    # ボールの位置を計算する
    ball.timeEvolution(deltaTime*@timeScale)

    # 軌跡の更新
    trail.update()

    # 速度/力ベクトルの更新
    velocityVector.setDirection ball.velocity.clone().normalize()
    velocityVector.setLength    ball.velocity.length()/4
    forceVector.setDirection    ball.force.clone().normalize()
    forceVector.setLength       ball.force.length()
    forceVector1.setDirection   ball.forces.tension.clone().normalize()
    forceVector1.setLength      ball.forces.tension.length()
    forceVector2.setDirection   ball.forces.gravity.clone().normalize()
    forceVector2.setLength      ball.forces.gravity.length()

    #ballMesh.position.copy ball.position
    #strap.geometry.vertices[1].copy ball.position
    strap.geometry.verticesNeedUpdate = true

  onWindowResize: ->
    camera.aspect = window.innerWidth/window.innerHeight
    camera.updateProjectionMatrix()
    renderer.setSize(window.innerWidth, window.innerHeight)
    trackball.handleResize()
    render()

#--------------------------------------------------------------------------------
#window.addEventListener "load", ->
#resizeTo(850, 610)
main = new Main()
main.start()
